; ===============================================================
; Mechanics
; ===============================================================

; ---------------------------------------------------------------
; CalcStat:
; Calculates a pokemon's stat
; Input: HL = pointer to poke ID
; 	  A = stat ID
; Output: HL = Stat value
; Stat IDs:
; 	0 = HP
;	1 = Atk
;	2 = Def
;	3 = Sp. Atk
;	4 = Sp. Def
;	5 = Spd
; ---------------------------------------------------------------

; ---------------------------------------------------------------
; Formulas:
; HP   = [IV + (2*Base) + EV/4 + 100] * Lvl / 100 + 10
;        [(2*Base) + EV/4 + 141]
; Stat = [[IV + (2*Base) + EV/4] * Lvl / 100 + 5] * Nature
;        [(2*Base) + EV/4 + 36] * Nature
; ---------------------------------------------------------------

CalcStat:
.relocate SwapZone
	push AF
	RCall( cGetPoke )
	push HL
	
	ld DE, 5
	add HL, DE
	call GetByte		; Item | EV Spread --> B
	inc HL
	call GetByteC		; EV Spread | Nature --> C
	ld B, A
	ld A, C
	srl B
	rr A
	srl B
	rr A
	srl B
	rr A
	srl A
	srl A
	ld B, A		; EV Spread --> B
	ld A, C
	and %00011111
	ld C, A		; Nature --> C
	
	pop HL
	push BC
	RCall( cGetName2 )
	pop BC
	pop AF
	
	push HL
	RCall( cCalcEV )	; EV --> B
	RCall( cCalcNature )	; Nature --> C
	pop HL
	
	push AF
	ld DE, 13
	add HL, DE
	ld D, 0
	ld E, A
	add HL, DE
	call GetByte		; Base --> A
	
	ld H, 0		; Stat --> HL
	ld L, A
	add HL, HL		; (Base*2)
	srl B			; (EV/4)
	srl B
	ld D, 0
	ld E, B
	add HL, DE
	pop AF
	
	or A
	jr nz, CalcStatNHP	; If Stat == HP:
	ld DE, 141		;   Value += 141
	add HL, DE
	ret		
CalcStatNHP:			; Else:
	ld DE, 36		;   Value += 36
	add HL, DE	
	
	ld A, C		; Nature --> A
	or A
	ret z
	
	push AF
	push HL
	ld D, 10
	RCall( cDivHL )	; HL/10 --> DE
	ld D, 0
	ld E, L
	pop HL
	pop AF
	
	cp 1
	jr nz, CalcStatDim	; If Boost
	add HL, DE		; HL = HL + HL/10
	ret
CalcStatDim:			; Else
	or A			; HL = HL - HL/10
	sbc HL, DE
	dec HL
	ret
.endrelocate
	
; ---------------------------------------------------------------
; CalcEV
; Given an EV spread and a stat, return the EV's of that stat
; Input: A = Stat
;	  B = EV Spread
; Output: B = EV (0, 168, 252)
; Preserves A, C
; ---------------------------------------------------------------

; ---------------------------------------------------------------
; EV Spread:
; --xxxxxx
; -------1 HP
; ------1- Attack
; -----1-- Defense
; ----1--- Special Attack
; ---1---- Special Defense
; --1----- Speed
; ---------------------------------------------------------------

CalcEV:
.relocate SwapZone
	ld H, $FF		; No stat --> H
	ld L, A		; Stat --> L
	ld D, 0		; 0 --> D (Total 1's in spread)
	ld E, 6		; 6 --> E (Counter)
CalcEVLp:
	srl B
	jr nc, CalcEVLpSkip	; If bit (6-E) of B == 1
	inc D			; then D++
	ld A, 6
	sub E
	cp L
	jr nz, CalcEVLpSkip	; If Stat has EV's
	ld H, 0		; Yes stat --> H
CalcEVLpSkip:
	dec E			; dec Counter
	jr nz, CalcEVLp
	ld A, H
	cp $FF			; If No stat...
	jr nz, CalcEVY	; ...then
	ld A, L		; Restore A
	ld B, 0		; 0 --> B
	ret			; and quit
CalcEVY:
	ld A, D
	cp 3
	jr nz, CalcEVNot1	; Elif D == 2
	ld A, L		; Restore A
	ld B, 168		; 168 --> B
	ret			; and quit
CalcEVNot1:
	ld A, L		; Else, 
	ld B, 252		; 252 --> B
	ret			; and quit
.endrelocate
	
; ---------------------------------------------------------------
; CalcNature:
; Determines whether nature boosts or diminishes stat
; Input: A = Stat
;	  C = Nature
; Output: C = Multiplier
;           = $FF (Diminish)
;           = 0 (Neutral)
;           = 1 (Boosts)
; Preserves A, B
; ---------------------------------------------------------------

; ---------------------------------------------------------------
;  -Atk  |  -Def  |  -SAtk  |  -SDef  |  -Spd   |
; -------|--------|---------|---------|---------|----------------
; Hardy  | Lonely | Adamant | Naughty | Brave   | +Atk
; Bold   | Docile | Impish  | Lax     | Relaxed | +Def
; Modest | Mild   | Bashful | Rash    | Quiet   | +SAtk
; Calm   | Gentle | Careful | Quirky  | Sassy   | +SDef
; Timid  | Hasty  | Jolly   | Naive   | Serious | +Spd
; ---------------------------------------------------------------

CalcNature:
.relocate SwapZone
	push AF
	ld A, C		; Nature / 5 + 1 = +Stat --> L
	ld D, 5		; Nature % 5 + 1 = -Stat --> H
	push BC
	RCall( cDiv )
	pop BC
	inc L
	inc A
	ld H, A
	cp L
	jr nz, CalcNatureNNeutral
	ld C, 0		; If Stat is + and -
	pop AF			; Neutral
	ret
CalcNatureNNeutral:
	pop AF
	cp H			; If Stat is -
	jr nz, CalcNatureNMin
	ld C, $FF		; Diminish
	ret
CalcNatureNMin:
	cp L			; If Stat is +
	jr nz, CalcNatureNPlus
	ld C, 1		; Boost
	ret
CalcNatureNPlus:
	ld C, 0		; Else
	ret			; Neutral
.endrelocate
	
; ---------------------------------------------------------------
; ChkGender
; Checks gender
; Input: HL = pointer to poke ID
; Output: A = 0 Genderless
;           = 1 Male
;           = 2 Female
; Preserves DE
; ---------------------------------------------------------------

; ---------------------------------------------------------------
; Base stat val  | Gender Ratio
;      255       | Genderless
;      254       | 100% Female
;      223       | 12.5% Male
;      191       | 25.0% Male
;      127       | 50.0% Male
;      63        | 75.0% Male
;      31        | 87.5% Male
;      0         | 100% Male
; http://bulbapedia.bulbagarden.net/wiki/Personality_value#Gender
; ---------------------------------------------------------------

ChkGender:
.relocate SwapZone
	push DE
	push HL
	inc HL
	inc HL
	ld B, (HL)		; get personality id --> B
	pop HL
	push BC
	RCall( cGetName )	; get pokedex data --> HL
	pop BC
	ld DE, 19		; get gender value --> A
	add HL, DE
	call GetByte
	pop DE
	cp 255			; if A = $FF
	jr z, ChkGenderNone	; ret (A = 0, Genderless)
	or A			; if A = 1
	jr z, ChkGenderMale	; ret (A = 1, Male)
	cp 254
	jr nz, ChkGenderSkip	; if 100% Female
	ld A, 2		; ret (A = 2, Female)
	ret
ChkGenderSkip:
	cp B			; if A < B
	jr c, ChkGenderMale	; then Male
ChkGenderFemale:
	ld A, 2		; else Female
	ret
	
ChkGenderMale:
	ld A, 1
	ret
	
ChkGenderNone:
	xor A
	ret
.endrelocate
	
; ---------------------------------------------------------------
; CalcPriority
; Determines move priority
; Input: A = Player move
;        B = Opp move
;        (PlayerPKMN), (OppPMKN), etc
; Output: c = player first, nc = opp first
; ---------------------------------------------------------------

; ---------------------------------------------------------------
; Turn priority pseudo-code
; ---------------------------------------------------------------
; If Player.move.priority > Opp.move.priority:
;    Player.attack()
; Elif Opp.move.priority > Player.move.priority:
;    Opp.attack()
; Else:
;       If Player.spd > Opp.spd:
;          Player.attack()
;       Elif Opp.spd > Player.spd:
;          Opp.attack()
;       Else:
;          If bit 0, Rand:
;             Player.attack()
;          Else:
;             Opp.attack()
; ---------------------------------------------------------------

CalcPriority:
.relocate SwapZone
	RCall( cCPPriority )
	or A
	jr nz, CalcPriorityNotEq
	
	ld A, (PlayerPKMN + PKMN_Item) 	; favors player if both pkmn have quick claw
	call CalcPriorityClawSub
	jr nc, CalcPriorityPlayerNotClaw
	ret c					; player moves first
	
CalcPriorityPlayerNotClaw:
	ld A, (OppPKMN + PKMN_Item)
	call CalcPriorityClawSub
	jr nc, CalcPriorityOppNotClaw
	or A					; opp moves first
	ret
	
CalcPriorityClawSub:				; A = hold item, outputs c if claw activated, nc otherwise
	cp iQuickClaw				; if holding quick claw
	scf
	ccf
	ret nz					; ret w/ nc if no quick claw
	ld B, 100
	call iRandom				; 0 <= A < 100
	cp 20					; 20% chance of activating
	ret					; ret w/ c or nc
	
CalcPriorityOppNotClaw:
	ld HL, PlayerPKMN
	RCall( cCalcSpd )		; HL = Player eff Spd
	push HL
	ld HL, OppPKMN
	RCall( cCalcSpd )
	ex DE, HL			; DE = Opp eff Spd
	pop HL
	call CpHLDE			; z if HL == DE, c if HL < DE
	jr nz, Turn_NotEq2
	
	call Rand127
	scf
	bit 0, A			; doesn't affect carry flag
	ret z
	ccf
	ret
	
Turn_NotEq2:
	ccf
	ret
	
CalcPriorityNotEq:
	cp 1
	jr nz, CalcPriorityOpp
CalcPriorityPlayer:
	scf
	ret
	
CalcPriorityOpp:
	scf
	ccf
	ret
.endrelocate
	
; ---------------------------------------------------------------
; CPPriority
; Compares move priority
; Input: A = Move 1
;	  B = Move 2
; Output: A = 0 if same
; 	       1 if Move 1 is higher
;		$FF if Move 2 is higher
; ---------------------------------------------------------------

; ---------------------------------------------------------------
; Priority Table
; +7 | Pursuit
; +6 | Switching out
; +3 | Detect, Endure, Follow Me, Protect
; +1 | ExtremeSpeed, Fake Out, Mach Punch, Quick Attack
; 0  | All other moves
; -3 | Focus Punch
; -5 | Counter, Mirror Coat, Roar
; http://bulbapedia.bulbagarden.net/wiki/Priority#Generation_III
; ---------------------------------------------------------------

CPPriority:
.relocate SwapZone
	cp $FD			; If player is switching out
	jr c, CPPriorityNoSwitch
	ld C, A
	ld A, B
	cp mPursuit		; and opponent is using pursuit
	ld A, C
	jr nz, CPPriorityNoPursuit
	ld A, $FF
	ret			; then ret w/ A = $FF, move 2 is faster
	; TODO: check pursuit for move 1
	
CPPriorityNoSwitch:
CPPriorityNoPursuit:
	push BC
	call CPPriorityGet	; Priority 1 --> A
	pop BC
	push AF
	ld A, B
	call CPPriorityGet	; Priority 2 --> B
	ld B, A
	pop AF
	
	cp B
	jr nz, CPPriorityNotEq
	xor A			; If Priority's equal
	ret			; return 0
	
CPPriorityNotEq:
	jr c, CPPriorityLess
	
	ld A, 1		; Else (P1 > P2)
	ret			; return 1
	
CPPriorityLess:
	ld A, $FF		; If P2 > P1:
	ret			; return $FF
	
CPPriorityGet: 		; subroutine - retrieves move priority, input: A = move, output: A = priority
	ld B, (CPPriorityData_End - CPPriorityData) / 2
	ld HL, CPPriorityData
CPPriorityGetLp:
	cp (HL)
	inc HL
	jr nz, CPPriorityGetSkip
	ld A, (HL)		; If Move == (HL)
	ret			; return (HL+1) (Priority)
	
CPPriorityGetSkip:
	inc HL
	djnz CPPriorityGetLp
	ld A, 7		; If no match
	ret			; return A = 7
	
CPPriorityData:		; Data format: (move, priority) (rel: -7)
.db $FF, 13, $FE, 13, $FD, 13, mDetect, 10, mEndure, 10
.db mFollowMe, 10, mProtect, 10, mExtremeSpeed, 8, mFakeOut, 8
.db mMachPunch, 8, mQuickAttack, 8, mFocusPunch, 4, mCounter, 2
.db mMirrorCoat, 2, mRoar, 2
CPPriorityData_End:
.endrelocate

; ---------------------------------------------------------------
; CalcMiss
; Calculates hit / miss status
; Input (AtkPKMN) = Attacking PKMN
;       (TargetPKMN) = Target PKMN
;       (AtkMove) = Attacking move
; Output: c = hit, nc = miss
;         (HitMiss) = 1 if hit, 0 if miss
; ---------------------------------------------------------------

CalcMiss:
.relocate SwapZone
	ld A, (AtkMove)
	RCall( cGetMoveEffect )	; A = move effect ID
	cp qNeverMiss			; if effect is never miss
	jr z, CalcMissStall		; then check stall conditions
	
	ld A, (AtkMove)
	RCall( cGetMoveAcc )		; A = move accuracy
	or A				; if acc is zero
	jr z, CalcMissHit2		; then automatically hit
	
CalcMissStall:
	ld HL, (TargetPKMN)
	call GetPKMNStall
	jr z, CalcMissNotStall	; skip if not stall move
	cp 2
	jr nz, CalcMissNotDig	; dig is hit by earthquake / fissure
	ld A, (AtkMove)
	cp mEarthquake
	jr z, CalcMissNotStall
	cp mFissure
	jr z, CalcMissNotStall
	jp CalcMissMiss
	
CalcMissNotDig:
	cp 3
	jr nz, CalcMissNotDive	; dive is hit by surf
	ld A, (AtkMove)
	cp mSurf
	jr z, CalcMissNotStall
	jr CalcMissMiss
	
CalcMissNotDive:			; bounce / fly is hit by thunder / sky uppercut
	ld A, (AtkMove)
	cp mThunder
	jr z, CalcMissNotStall
	cp mSkyUppercut
	jr z, CalcMissNotStall
	jr CalcMissMiss
	
CalcMissNotStall:
	ld HL, (TargetPKMN)
	call GetPKMNAbility
	cp aLevitate
	jr nz, CalcMissNotLevitate	; if ability is levitate
	ld A, (AtkMove)		; A = move ID
	RCall ( cGetMoveType )	; A = move type ID
	cp eGround			; and move is ground type
	jr nz, CalcMissNotLevitate	; then miss
	ld A, 1
	ld (LevitateAct), A		; set LevitateAct flag
	jr CalcMissMiss
	
CalcMissNotLevitate:
	ld A, (AtkMove)
	RCall( cCalcMoveAcc )	; A = move accuracy
	
	or A
CalcMissHit2:
	jr z, CalcMissHit		; if acc == 0, then hit
	
	ld D, 0
	ld E, A
	ld HL, (AtkPKMN)
	ld BC, PKMM_AccBoost
	add HL, BC
	ld A, (HL)
	RCall( cCalcMultAcc )
	ex DE, HL			; DE = acc * acc_mod
	
	ld HL, (TargetPKMN)
	push HL
	ld BC, PKMN_EvaBoost
	add HL, BC
	ld A, (HL)
	neg
	RCall( cCalcMultAcc )
	ex DE, HL			; DE = DE / eva_mod
	pop HL
	
	; sand veil - add 20% evasion in sandstorm
	; TODO: sand veil gives immunity to sandstorm damage?
	call CalcMissSandVeil
	
	; brightpowder - add 10% evasion
	call GetPKMNItem		; if brightpowder, then add 10% evasion; preserves DE
	cp iBrightpowder
	ex DE, HL
	jr nz, CalcMissNotBrightpowder
	
	; TODO: lightning attack is 100% acc w/ lightningod?
	
	call Calc10
	or A				; clear c flag
	sbc HL, DE			; HL = HL - HL / 10
	
CalcMissNotBrightpowder:
	ld A, H
	or A				; if HL >= 256
	jr nz, CalcMissHit		; then hit
	ld A, L
	cp 100				; if HL >= 100
	jr nc, CalcMissHit		; then hit
	ld B, A
	
	push BC
	ld B, 100
	call iRandom			; A = rand, 0 <= A < 100
	pop BC
	cp B				; if A < B, then hit
	jr c, CalcMissHit		; else, miss
CalcMissMiss:
	ld HL, HitMiss
	ld (HL), 0
	or A				; clear c flag
	ret
	
CalcMissHit:
	ld HL, HitMiss
	ld (HL), 1
	scf
	ret
	
CalcMissSandVeil:			; subroutine - calculates effect of sand veil on acc, input: HL = pt to data, DE = acc, output: DE = acc
	call GetPKMNAbility		
	cp aSandVeil
	ret nz
	ld A, (Weather)
	cp wSandstorm
	ret nz
	
	push HL			; save HL, pt to data
	ex DE, HL			; HL = acc
	call Calc20
	or A				; clear c flag
	sbc HL, DE			; HL = HL - HL / 5
	ex DE, HL			; DE = HL
	pop HL				; get HL, pt to data
	ret
.endrelocate

; ---------------------------------------------------------------
; CalcMoveAcc
; Calculates move accuracy, takes weather into account
; Input: A = atk move
; Output: A = move accuracy (0 = no miss)
; ---------------------------------------------------------------

CalcMoveAcc:
	call ChkCloudNine		; nc = pkmn has cloud nine
	jr nc, CalcMoveAccNotThunder
	
	cp mThunder
	jr nz, CalcMoveAccNotThunder
	
	ld B, A			; save A in B
	ld A, (Weather)
	cp wSun
	jr z, CalcMoveAcc50		; thunder has 50% acc in sun
	cp wRain
	jr z, CalcMoveAcc0		; thunder has perfect acc in rain
	ld A, B			; restore A from B
	
CalcMoveAccNotThunder:
	RCall( cGetMoveAcc )
	ret
	
CalcMoveAcc50:
	ld A, 50
	ret				; ret w/ acc = 50
	
CalcMoveAcc0:
	xor A
	ret				; ret w/ acc = 0
	
; ---------------------------------------------------------------
; CalcDamage
; Calculates damage
; Input: (AtkPKMN) = Attacking PKMN
; 	  (TargetPKMN) = Target PKMN
;	  (AtkMove) = Attacking Move
; Output: HL = Damage
;	   nc = No damage
;         A = 0 if damage | 1 if no pow | 2 if not affect
; ---------------------------------------------------------------

; ---------------------------------------------------------------
; Damage = ((((42 * A * P ) / D) / 50) + 2) * Mod
; Mod = T * C * STAB * Other * R / 255
; 217 <= R <= 255
; ---------------------------------------------------------------

CalcDamage:				; ~ 235 bytes
.relocate SwapZone
	ld A, (AtkMove)
	ld HL, (TargetPKMN)
	RCall( cCalcEff )
	or A
	jp z, CalcDamage00
	
	ld A, (AtkMove)
	RCall( cGetMoveEffect )	; A = move effect
	cp qOHKO
	jp z, CalcDamageOHKO
	cp qLevelDmg
	jp z, CalcDamageLevel
	
	ld HL, (AtkPKMN)		; determines critical hit
	ld A, (AtkMove)
	RCall( cCalcCrit )
	
	ld HL, FP42			; 42 --> OP2
	rst rMov9ToOP1
	rst rOP1ToOP2
	
	ld HL, (AtkPKMN)
	ld A, (AtkMove)
	RCall( cCalcAtk )
	call CalcDamageMult		; OP2 *= A
	
	ld A, (AtkMove)
	RCall( cCalcPow )
	ld A, L
	or H				; if HL == 0
	jr z, CalcDamage0		; do no damage
	call CalcDamageMult		; OP2 *= P
	
	ld HL, (TargetPKMN)
	ld A, (AtkMove)
	RCall( cCalcDef )
	RCall( cIntFP )
	RCall( cSwapOP1OP2 )
	b_call(_FPDiv)		; OP1 = OP2 / D
	
	ld HL, FP50
	b_call(_Mov9ToOP2)		; OP1 /= 50
	b_call(_FPDiv)
	
	b_call(_Plus1)
	b_call(_Plus1)
	rst rOP1ToOP2			; OP2 = OP1 + 2
	
	RCall( cCalcEffDam )
	b_call(_FPMult)
	rst rOP1ToOP2			; OP2 *= T
	
	RCall( cCalcCritDam )
	call CalcDamageMult		; OP2 *= C
	
	ld A, (AtkMove)
	ld HL, (AtkPKMN)
	RCall( cCalcSTAB )
	b_call(_FPMult)
	rst rOP1ToOP2			; OP2 *= STAB
	
	RCall( cCalcRand )
	RCall( cIntFP )
	b_call(_FPMult)		; OP1 = OP2 * R, 217 <= R <= 255
	
	ld HL, FP255
	b_call(_Mov9ToOP2)
	b_call(_FPDiv)		; OP1 /= 255
	
	call ChkOPErr
	b_call(_ConvOp1)		; DE = OP1, error if OP1 exponent > 3
	RCall( cCalcEndure )		; calculates effect of endure, outputs to DE
	RCall( cCalcFocusBand )	; calculates effect of focus band, outputs to DE
	ex DE, HL			; HL = OP1
	
CalcDamageSub:			; store HL in (Damage) var, ret w/ A = 0 and c flag set
	xor A				; A = 0, dam
	ld (Damage), HL
	ld (DamageEff), A
	scf
	ret
	
CalcDamageOHKO:
	ld HL, (TargetPKMN)
	call GetPKMNMaxHP		; damage = maxHP
	jr CalcDamageSub
	
CalcDamageLevel:
	ld HL, 100			; damage = level = 100
	jr CalcDamageSub
	
CalcDamage0:				; move pow = 0
	ld A, 1			; A = 1, no pow
	jr CalcDamage00Sub
	
CalcDamage00:				; type eff = not affect
	ld A, 2			; A = 2, no eff
CalcDamage00Sub:
	ld HL, 0
	ld (Damage), HL
	ld (DamageEff), A
	or A
	ret
	
CalcDamageMult:			; subroutine - multiplies OP2 by HL, stores result in OP2, input: HL = int, output: OP2 = OP2 * FP(HL)
	RCall( cIntFP )		; OP1 = FP(HL)
	b_call(_FPMult)		; OP1 = OP1 * OP2
	rst rOP1ToOP2			; OP2 = OP1
	ret				; ret with OP2
.endrelocate

; ---------------------------------------------------------------
; CalcAtk
; Calculates effective attack of PKMN
; Takes into account raw attack, phys/spec, stat mod, 
; Input: HL = pointer to detailed data
;        A  = attack ID
; Output: HL = Atk
; ---------------------------------------------------------------

CalcAtk:
.relocate SwapZone
	push HL
	push HL
	RCall( cGetMoveType ); A = atk type
	pop HL
	cp eThreshold
	jr c, CalcAtkPhys	; if type < thres, then phys
CalcAtkSpec:			; else, spec
	ld BC, PKMN_SAtk
	jr CalcAtkGet
	
CalcAtkPhys:
	ld BC, PKMN_Atk
CalcAtkGet:
	add HL, BC
	push HL
	call LdHLIndRAM	; BC not affected
	ex DE, HL		; DE = phys/spec atk
	pop HL
	
CalcAtkMod:
	push BC
	ld A, C		; A = SAtk | Atk offset, offset is 8-bit value
	cp PKMN_Atk
	ld BC, PKMN_SAtkBoost - PKMN_SAtk
	jr nz, CalcAtkBoostNotPhys
	ld BC, PKMN_AtkBoost - PKMN_Atk
	
CalcAtkBoostNotPhys:
	add HL, BC		; get boost pointer
	ld A, (HL)		; A = phys/spec atk boost
	cp 7			; if A >= 7, atk boost is negative
	jr c, CalcAtkBoostNNeg
	
	ld B, A		; save atk boost as B
	ld A, (CritBool)	; A = whether attack is critical hit
	or A			; if A != 0, attack is crit
	ld A, B		; restore atk boost
	jr nz, CalcAtkBoostSkip	; then skip boost multiplier
	
CalcAtkBoostNNeg:
	RCall( cCalcMult )
	
CalcAtkBoostSkip:
	ex DE, HL		; DE = mult'ed phys/spec atk
	pop BC
	pop HL
	
	ld A, C		; A = SAtk | Atk offset, offset is 8-bit value
	push HL		; save HL
	call GetLight		; HL = pt to light screen status of target pkmn, AF preserved
	ld B, (HL)		; B = light screen status
	inc HL			; Reflect - Light = 1
	ld C, (HL)		; C = reflect status
	pop HL			; get HL, pt to data
	cp PKMN_Atk
	jr nz, CalcAtkNotPhys
	
	ld A, C		; A = reflect status
	or A
	jr z, CalcAtkNotReflect
	
	ld A, (CritBool)
	or A			; if critical hit, then reflect has no effect
	jr nz, CalcAtkNotReflect
	
	srl D			; if phys atk and reflect up
	rr E			; then DE /= 2
	
CalcAtkNotReflect:
	call GetPKMNItem	; A = item, preserves HL, destroys BC
	cp iChoiceBand	; if holding choice band and using phys attack
	jr z, CalcAtkChoice	; then go to choice
	cp iThickClub		; else if holding thick club and using phys attack
	jr z, CalcAtkClub
	jr CalcAtkCont
	
CalcAtkChoice:		; if CB
	call Calc1p5		; then DE = 1.5 * DE, preserves HL
	jr CalcAtkCont
	
CalcAtkClub:			; thick club doubles attack, assumes holder is Marowak
	sla E
	rl D			; DE = DE * 2
	jr CalcAtkCont
	
CalcAtkNotPhys:
	ld A, B		; A = light screen status
	or A
	jr z, CalcAtkNotLight
	
	ld A, (CritBool)
	or A			; if critical hit, then light screen has no effect
	jr nz, CalcAtkNotLight
	
	srl D			; if spec atk and light screen up
	rr E			; then DE /= 2
	
CalcAtkNotLight:
CalcAtkCont:
	call GetPKMNAbility	; A = pkmn ability, preserves HL, DE
	cp aGuts
	jr z, CalcAtkGuts
	
	call GetPKMNStatus	; A = pkmn status, preserves HL, DE
	cp stBRN
	jr nz, CalcAtkNotBRN
	
	srl D			; if status == BRN and ability != guts
	rr E			; then DE /= 2
	jr CalcAtkNotGuts
	
CalcAtkGuts:
	call GetPKMNStatus	; A = pkmn status
	or A			; if A = 0, stNone
	jr z, CalcAtkNotGuts	; then skip
	
	call Calc1p5		; DE = DE * 1.5
	
CalcAtkNotBRN:
CalcAtkNotGuts:
	call GetPKMNAbility	; A = ability, preserves HL, DE
	ex DE, HL		; HL = atk
	cp aPurePower		; if ability not pure power
	ret nz			; then ret
	
	sla L			; else
	rl H			; HL *= 2
	ret
.endrelocate

; ---------------------------------------------------------------
; CalcPow
; Calculates effective power
; Input: A = Move ID
;        (AtkPKMN) = Current attacking PKMN
; Output: HL = Power
; Preserves OP2, destroys OP1, OP3, OP4 (in case of reversal)
; ---------------------------------------------------------------

CalcPow:				; ~ 232 bytes
.relocate SwapZone
	push AF			; save move ID
	call CalcPowMove		; calculates power, given move
	call CalcPowEff		; calculates power, given move effect
	pop AF				; get A = move ID
	
	push AF			; save move ID
	call CalcPowItem		; calculates power, given item
	pop AF				; get A = move ID
	
	push AF			; save move ID
	RCall( cCalcPowAbil )	; calculates power, given ability
	pop AF				; get A = move ID
	
	RCall( cCalcPowWeather )	; calculates power, given weather
	ret
	
CalcPowMove:				; subroutine - calculates power, given move, input: A = move ID, output: HL = power, saves A
	push AF			; save move ID
	RCall( cGetMovePow )		; A = move pow
	ld H, 0
	ld L, A			; HL = move pow
	pop AF				; get move ID
	ret
	
CalcPowItem:				; subroutine - calculates power, given item, input: HL = pow, A = move ID, output: HL = pow
	push HL			; save move pow
	push AF			; save A = move ID
	ld HL, (AtkPKMN)
	call GetPKMNItem
	ld B, A			; B = PKMN Item
	pop AF				; A = Move ID
	
	push BC
	RCall( cGetMoveType )	; A = Type ID
	pop BC
	pop HL				; get move pow
	
	push HL			; save HL, move pow
	ld D, B
	ld E, A			; DE = B * 256 + A
	
	ld B, 9			; 9 type boosting items total
	ld HL, CalcPowTypeDb
CalcPowTypeLp:
	push BC
	push HL
	call LdHLIndRAM		; HL = (HL), H = (HL + 1), L = (HL)
	call CpHLDE			; preseves regs, compare (H == D && L == E)
	pop HL
	inc HL
	inc HL				; HL += 2
	pop BC				; pop / inc HL preserve z flag
	jr nz, CalcPowTypeBoost
	djnz CalcPowTypeLp
	
	pop HL
	ret
	
CalcPowTypeBoost:	; TODO: debug
	pop HL
	call Calc10			; DE = HL / 10
	add HL, DE			; HL = HL + HL / 10
	ret
	
CalcPowEff:				; subroutine - calculates power (HL), given move ID (A)
	push HL			; save HL
	RCall( cGetMoveEffect )	; A = move effect ID
	pop HL				; get HL
	
	cp qReverse
	jr nz, CalcPowNotReversal
	
	RCall( cCalcPowReversal )	; returns HL = pow
	ret
	
CalcPowNotReversal:
	cp qFrustration
	jr nz, CalcPowNotFrustration
	
	; Frustration - pow = 102, b/c friendship = 0
	ld HL, 102
	ret
	
CalcPowNotFrustration:
	cp qRollout
	jr nz, CalcPowNotRollout
	
	; Rollout - pow doubles for each consecutive turn
	push HL			; save HL, move pow
	call PKMNGetRollout		; HL = pt to rollout status of (AtkPKMN)
	ld B, (HL)			; B = number of turns in rollout
	inc HL				; PKMN_DefCurl - PKMN_Rollout = 1
	ld A, (HL)			; A = defense curl status
	pop HL				; get HL, move pow
	or A				; if defense curl is set
	jr z, CalcPowRolloutNotDefCurl
	inc B				; then inc B, to double pow
	
CalcPowRolloutNotDefCurl:
	ld A, B
	or A
	ret z				; ret if no boosts (rollout = 0 and defcurl = 0)
	
CalcPowRolloutLp:			; HL = move pow, B = num boosts
	sla L
	rl H				; HL = HL * 2
	djnz CalcPowRolloutLp
	ret
	
CalcPowNotRollout:
	cp qFacade
	jr nz, CalcPowNotFacade
	
	; Facade - pow doubles if user has burn / paralysis / poison
	push HL			; save move pow
	ld HL, (AtkPKMN)
	call GetPKMNStatus		; A = status of (AtkPKMN)
	pop HL				; get move pow
	cp stBRN
	jr z, CalcPowFacadeBRN
	cp stPAR
	jr z, CalcPowFacadePAR
	cp stPSN
	jr z, CalcPowFacadePSN
	cp stBADPSN
	jr z, CalcPowFacadeBADPSN
	ret				; ret if no status conditions
	
CalcPowFacadeBRN:
CalcPowFacadePAR:
CalcPowFacadePSN:
CalcPowFacadeBADPSN:			; if (BRN | PAR | PSN | BADPSN)
	sla L
	rl H				; then HL = HL * 2
	ret
	
CalcPowNotFacade:
	cp qSmellingsalt
	jr nz, CalcPowNotSmellingsalt
	
	; Smellingsalt - pow doubles if target is paralyzed
	push HL			; save move pow
	ld HL, (TargetPKMN)
	call GetPKMNStatus		; A = status of (TargetPKMN)
	pop HL				; get move pow
	cp stPAR			; if not paralyzed
	ret nz				; then skip
	
	sla L
	rl H
	ret				; else, ret w/ HL = HL * 2
	
CalcPowNotSmellingsalt:
	cp qPursuit
	jr nz, CalcPowNotPursuit
	
	; Pursuit - pow doubles if target is switching out
	ld A, (TargetMove)
	cp $FD
	jr c, CalcPowNotPursuit	; skip if target not switching out
	sla L
	rl H				; else, ret w/ HL = HL * 2
	ret
	
CalcPowNotPursuit:
	cp qRevenge
	ret nz
	
	; Revenge - pow doubles if user takes damage in turn
	ld A, (TurnNum)
	cp 2
	ret nz				; ret if not second turn
	
	ld DE, (LastDmg)		; DE = damage done in last turn
	ld A, E
	or D
	ret z				; ret if 0 damage
	
	sla L				; otherwise, if user took damage
	rl H				; then HL = HL * 2
	ret
	
CalcPowTypeDb:			; database of item - type boost relations, 9 * 2 bytes long
.db eFire, iCharcoal, eElectric, iMagnet, eGrass, iMiracleSeed, eWater, iMysticWater, eIce, iNevermeltice
.db ePoison, iPoisonbarb, eFlying, iSharpbeak, eGround, iSoftsand, ePsychic, iTwistedspoon
.endrelocate

; ---------------------------------------------------------------
; CalcPowReversal
; Calculates power of reversal, based on current HP
; Input: (AtkPKMN) = pkmn using reversal
; Output: HL = power
; Preserves OP2, destroys OP1, OP3, OP4
; ---------------------------------------------------------------

; ---------------------------------------------------------------
; Reversal Power Calculation:
; N = floor( 64 * current HP / total HP )
; 0  - 1  | 200
; 2  - 5  | 150
; 6  - 12 | 100
; 13 - 21 | 80
; 22 - 42 | 40
; 43 - 64 | 20
; ---------------------------------------------------------------
	
CalcPowReversal:
.relocate SwapZone
	b_call(_OP2ToOP1)
	b_call(_PushOP1)		; save OP2 in stack
	ld HL, (AtkPKMN)
	push HL
	call GetPKMNHP		; HL = current HP
	RCall( cIntFP )		; OP1 = current HP
	rst rOP1ToOP2			; OP2 = current HP
	pop HL
	call GetPKMNMaxHP		; HL = max HP
	RCall( cIntFP )		; OP1 = max HP
	b_call(_OP1ToOP4)		; OP4 = max HP
	ld HL, FP64
	rst rMov9ToOP1		; OP1 = 64
	b_call(_FPMult)		; OP1 = 64 * current HP, uses OP3
	b_call(_OP4ToOP2)		; OP2 = max HP
	b_call(_FPDiv)		; OP1 = 64 * current HP / max HP
	b_call(_ConvOp1)		; DE = OP1, D should be 0
	push DE
	b_call(_PopOP1)
	rst rOP1ToOP2			; retrieve OP2
	pop DE
	ld A, E			; A = E, 0 <= A <= 64
	
	cp 2				; if A < 2
	jr c, CalcPowRev200		; then pow = 200
	cp 6				; else if A < 6
	jr c, CalcPowRev150		; then pow = 150
	cp 13				; else if A < 13
	jr c, CalcPowRev100		; then pow = 100
	cp 22				; else if A < 22
	jr c, CalcPowRev80		; then pow = 80
	cp 43				; else if A < 43
	jr c, CalcPowRev40		; then pow = 40
CalcPowRev20:				; else pow = 20
	ld HL, 20
	ret
	
CalcPowRev40:
	ld HL, 40
	ret
	
CalcPowRev80:
	ld HL, 80
	ret
	
CalcPowRev100:
	ld HL, 100
	ret
	
CalcPowRev150:
	ld HL, 150
	ret
	
CalcPowRev200:
	ld HL, 200
	ret
.endrelocate

; ---------------------------------------------------------------
; CalcPowAbil
; Calculates effect of ability on power
; Input: A = move ID
;        HL = pow
; Output: HL = pow
; ---------------------------------------------------------------

CalcPowAbil:				; subroutine - calculates power, given ability, input: A = move ID, HL = pow, output: HL = pow
.relocate SwapZone
	; thick fat
	push AF			; save A, move ID
	push HL			; save HL, pow
	ld HL, (TargetPKMN)		; HL = pt to target pkmn
	call GetPKMNAbility		; A = ability, saves HL, DE
	pop HL				; get HL, pow
	pop DE				; get D, move ID
	cp aThickFat			; if ability not thick fat, then skip
	ld A, D			; restore A, move ID
	jr nz, CalcPowAbilNotThickFat
	
	push HL			; save HL, pow
	push DE			; save D, move ID
	RCall( cGetMoveType )	; A = type ID of move
	pop DE				; get D, move ID
	pop HL				; get HL, pow
	cp eIce			; if ice move
	jr z, CalcPowAbilIceFire	; then activate ability
	cp eFire			; else if fire move
	jr z, CalcPowAbilIceFire	; then activate ability
	jr CalcPowAbilNotIceFire	; else, restore A and skip
	
CalcPowAbilIceFire:
	srl H				; if ability = thick fat and move is ice or fire type
	rr L				; then HL /= 2
	
CalcPowAbilNotIceFire:
	ld A, D			; restore A, move ID
	
CalcPowAbilNotThickFat:
	; flash fire
	push AF			; save A, move ID
	push HL			; save HL, pow
	ld HL, (AtkPKMN)
	ld BC, PKMN_FlashFire
	add HL, BC
	ld A, (HL)
	or A				; if flash fire not activated
	pop HL				; get HL, pow
	pop BC
	ld A, B			; get A, move ID
	jr z, CalcPowAbilNotFlashFire ; then skip
	
	push AF			; save A, move ID
	push HL			; save HL, pow
	RCall( cGetMoveType )	; A = move type
	cp eFire			; if attacking move is not fire type
	pop HL				; get HL, pow
	pop BC
	ld A, B			; get A, move ID
	jr nz, CalcPowAbilNotFlashFire ; then skip
	
	call Calc50			; else, DE = HL / 2
	add HL, DE			; HL = HL + HL / 2, increase pow by 50%
	
CalcPowAbilNotFlashFire:
	; blaze, overgrow, swarm, torrent
	push AF			; save A, move ID
	push HL			; save HL, pow
	ld HL, (AtkPKMN)		; HL = pt to atk pkmn
	call ChkThirdHP		; check if HP < 1/3 max HP, saves HL
	pop HL				; get HL, pow
	pop DE				; get D, move ID
	ret nc				; ret if HL >= DE
	
	push HL			; save HL, pow
	ld HL, (AtkPKMN)		; HL = pt to atk pkmn
	call GetPKMNAbility		; A = ability, saves HL, DE
	pop HL
	
	ld E, eFire
	cp aBlaze			; if ability is blaze
	jr z, CalcPowAbilChk		; then activate
	ld E, eGrass
	cp aOvergrow			; if ability is overgrow
	jr z, CalcPowAbilChk		; then activate
	ld E, eBug
	cp aSwarm			; if ability is swarm
	jr z, CalcPowAbilChk		; then activate
	ld E, eWater
	cp aTorrent			; if ability is not torrent
	ret nz				; then ret, else activate
	
CalcPowAbilChk:			; check if move is given type = E
	push DE
	push HL
	ld A, D			; get A, move ID
	RCall( cGetMoveType )	; A = type ID of move
	pop HL
	pop DE
	cp E				; if A != E
	ret nz				; then ret, types don't match
	
	call Calc50			; else, DE = HL / 2
	add HL, DE			; HL = HL + HL / 2, increase pow by 50%
	ret
.endrelocate

; ---------------------------------------------------------------
; CalcPowWeather
; Calculates effect of weather on power
; Input: A = move ID
;        HL = pow
; Output: HL = pow
; ---------------------------------------------------------------

CalcPowWeather:			; subroutine - calculates power, given weather, input: A = move ID, HL = pow, output: HL = pow
.relocate SwapZone
	call ChkCloudNine		; if pkmn on field has cloud nine
	ret nc				; then ret, weather has no effect
	
	cp mSolarBeam			; if move is solar beam
	jr nz, CalcPowWeatherNotSolarbeam
	
	push AF			; save A, move ID
	call CalcPowWeatherSolarbeam; then cut power by 50% if weather is rain, sandstorm or hail
	pop AF				; get A, move ID
	
CalcPowWeatherNotSolarbeam:
	push HL
	RCall( cGetMoveType )	; A = move type
	ld B, A			; B = move type
	pop HL
	
	call Calc50			; DE = HL / 2
	ld A, (Weather)
	cp wSun			; if weather is sunny
	jr z, CalcPowWeatherSun	; then goto sun
	cp wRain			; else if weather is rainy
	jr z, CalcPowWeatherRain	; then goto rain
	ret				; ret if not sun or rain
	
CalcPowWeatherSun:
	ld A, B			; A = move type
	cp eFire			; if sunny and fire move
	jr z, CalcPowWeatherAdd	; then boost
	cp eWater			; if sunny and water move
	jr z, CalcPowWeatherCut	; then diminish
	ret				; otherwise, ret
	
CalcPowWeatherRain:
	ld A, B			; A = move type
	cp eFire			; if rainy and fire move
	jr z, CalcPowWeatherCut	; then diminish
	cp eWater			; if rainy and water move
	jr z, CalcPowWeatherAdd	; then boost
	ret				; otherwise, ret
	
CalcPowWeatherAdd:			; adds 50% to pow
	add HL, DE
	ret
	
CalcPowWeatherCut:			; subs 50% from pow
	or A
	sbc HL, DE
	ret
	
CalcPowWeatherSolarbeam:		; subroutine - cuts pow by 50% if weather is rain, sandstorm or hail, input: HL = pow, output: HL = pow
	ld A, (Weather)
	or A
	ret z				; ret if no weather (wNone)
	dec A
	ret z				; ret if sunny (wSun)
	
	srl H				; HL = HL / 2
	rr L
	ret
.endrelocate

; ---------------------------------------------------------------
; CalcDef
; Calculates effective defense
; Takes into account raw def, phys/spec, stat mult, explosion
; Input: HL = pointer to detailed data
;        A = attack ID
; Output: HL = Def
; ---------------------------------------------------------------

CalcDef:
.relocate SwapZone
	push AF		; save attack ID
	push HL
	RCall( cGetMoveType ); A = atk type
	pop HL
	cp eThreshold
	jr c, CalcDefPhys	; if type < thres, then phys
CalcDefSpec:			; else, spec
	ld BC, PKMN_SDef
	jr CalcDefGet
	
CalcDefPhys:
	ld BC, PKMN_Def
CalcDefGet:
	add HL, BC
	push HL
	call LdHLIndRAM
	ex DE, HL		; DE = phys/spec def
	pop HL
	pop AF
	
	; marvel scale - if PAR/PSN/BRN/FRZ/SLP, then inc def by 50%
	call GetPKMNAbility	; A = pkmn ability
	cp aMarvelScale
	jr nz, CalcDefNotScale
	call GetPKMNStatus	; A = pkmn status
	or A			; compare to 0, stNone
	jr z, CalcDefNotScale
	
	call Calc1p5		; DE = DE * 1.5, preserves HL
	
CalcDefNotScale:
	cp mExplosion		; if move = explosion
	jr nz, CalcDefNotExplosion
	
	srl D
	rr E			; DE /= 2
	
CalcDefNotExplosion:
CalcDefMod:
	ld BC, PKMN_DefBoost - PKMN_Def
	add HL, BC
	ld A, (HL)		; A = phys/spec def boost
	cp 7			; if A < 7, def boost is positive
	jr nc, CalcDefModNPos
	
	ld B, A		; save def boost as B
	ld A, (CritBool)	; A = whether attack is critical hit
	or A			; if A != 0, attack is crit
	jr nz, CalcDefCrit	; then return DE, without applying multiplier
	ld A, B		; restore def boost as B
	
CalcDefModNPos:
	RCall( cCalcMult )
	ret
	
CalcDefCrit:
	ex DE, HL		; HL = def
	ret
.endrelocate
	
; ---------------------------------------------------------------
; CalcSpd
; Calculates effective speed
; Takes into account, raw spd, stat mult,
; Input: HL = pointer to detailed data
; Output: HL = Spd
; ---------------------------------------------------------------

CalcSpd:
.relocate SwapZone
	push HL
	push HL
	call GetPKMNSpd	; HL = spd
	ex DE, HL		; DE = spd
	pop HL
	
CalcSpdMod:
	ld BC, PKMN_SpdBoost
	add HL, BC
	ld A, (HL)		; A = pkmn spd boost
	RCall( cCalcMult )	; HL = spd w/ boost factored in
	ex DE, HL		; DE = boosted spd
	pop HL
	
	call GetPKMNStatus	; A = status ailment, HL, DE not affected
	cp stPAR
	jr nz, CalcSpdNotPAR
	
	srl D			; if status == PAR
	rr E			; then DE /= 4
	srl D
	rr E
CalcSpdNotPAR:
	call GetPKMNAbility	; A = ability, HL, DE not affected
	cp aChlorophyll	; if not chlorophyll
	jr nz, CalcSpdNotChloro; then skip
	
	ld A, (Weather)	; A = weather
	cp wSun		; if not sunny
	jr nz, CalcSpdNotChloro; then skip
	
	sla E			; else
	rl D			; DE *= 2
CalcSpdNotChloro:
	call GetPKMNAbility	; A = ability, HL, DE not affected
	ex DE, HL		; HL = boosted spd
	cp aSwiftSwim		; if not swift swim
	ret nz			; then ret
	
	ld A, (Weather)	; A = weather
	cp wRain		; if not rainy
	ret nz			; then ret
	
	sla L			; else
	rl H			; HL *= 2
	ret
.endrelocate
	
; ---------------------------------------------------------------
; CalcMult
; Calculates stat w/ multiplier
; Input: DE = stat
;        A = multiplier
; Output: HL = stat w/ mult
; ---------------------------------------------------------------

; ---------------------------------------------------------------
; Stat multiplier chart
; [Boost]	Multiplier
; ---------------------------------------------------------------
; +6		8/2
; +5		7/2
; +4		6/2
; +3		5/2
; +2		4/2
; +1		3/2
; 0		2/2
; -1		2/3
; -2		2/4
; -3		2/5
; -4		2/6
; -5		2/7
; -6		2/8
; ---------------------------------------------------------------

CalcMult:
.relocate SwapZone
	ex DE, HL		; HL = stat
	cp 7			; if A >= 7 (not btwn 0-6)
	jr nc, CalcMultLower	; then lower stat
CalcMultBoost:		; else boost stat
	or A
	ret z			; return if no stat boost, stat is same
	
	srl H			; HL /= 2, HL must be less than 510
	rr L
	inc A
	inc A
	ld H, A
	call MultHL		; HL = HL * (A + 2)
	ret
	
CalcMultLower:		; DE must be less than 765
	ld B, A
	ld A, 2
	sub B
	ld D, A
	RCall( cDivHL )	; HL = HL / (2 - A)
	xor A
	ld H, A
	sla L
	rl H			; HL *= 2
	ret
.endrelocate
	
; ---------------------------------------------------------------
; CalcMultAcc
; Calculates accuracy/evasion stat w/ multiplier
; Input: DE = accuracy
;        A = multiplier
; Output: HL = stat w/ mult
; ---------------------------------------------------------------

; ---------------------------------------------------------------
; Stat multiplier chart
; [Boost]	Multiplier
; ---------------------------------------------------------------
; +6		9/3
; +5		8/3
; +4		7/3
; +3		6/3
; +2		5/3
; +1		4/3
; 0		3/3
; -1		3/4
; -2		3/5
; -3		3/6
; -4		3/7
; -5		3/8
; -6		3/9
; ---------------------------------------------------------------

CalcMultAcc:
.relocate SwapZone
	cp 7			; if A >= 7 (not btwn 0-6)
	jr nc, CalcMultAccLower; then lower acc
CalcMultAccBoost:		; else boost acc
	or A
	jr z, CalcMultAccSame
	
	ex DE, HL
	ld D, 3
	RCall( cDivHL )	; L = DE / 3, DE must be less than 765
	
	inc A
	inc A
	inc A
	ld H, A
	call MultHL		; HL = DE * (A + 3)
	ret
	
CalcMultAccLower:		; DE must be less than 1020
	ex DE, HL		; HL = DE
	ld B, A
	ld A, 3
	sub B
	ld D, A
	RCall( cDivHL )	; L = HL / (3 - A)
	ld H, 3
	call MultHL		; HL = L * 3
	ret
	
CalcMultAccSame:
	ex DE, HL
	ret
.endrelocate
	
; ---------------------------------------------------------------
; CalcCrit
; Calculates whether attack is a critical hit 
; Input: HL = pointer to detailed data
;        A = attack ID
; Output: (CritBool) = critical hit status
; TODO: atk neg stats, def pos stats
; ---------------------------------------------------------------

; ---------------------------------------------------------------
; Critical hit probabilities
; Stage | Percentage | Fraction
; 1     | 6.25%      | 1/16 or 3/48
; 2     | 12.5%      | 1/8  or 6/48
; 3     | 25%        | 1/4  or 12/48
; 4     | 33.3%      | 1/3  or 16/48
; 5+    | 50%        | 1/2  or 24/48
; ---------------------------------------------------------------
; HCHR -> +2 stage, Scope Lens -> +1 stage, Blaze -> +1 stage
; ---------------------------------------------------------------

CalcCrit:
.relocate SwapZone
	cp mHitInConfusion  		; if hit self in confusion
	jr z, CalcCritNo		; then no crit
	cp mFutureHit			; if future sight hit
	jr z, CalcCritNo		; then no crit
	
	call GetPKMNAbility		; A = pkmn ability, saves HL
	cp aBattleArmor		; if ability is battle armor
	jr z, CalcCritNo		; then no crit
	cp aShellArmor      		; if ability is shell armor
	jr z, CalcCritNo		; then no crit
	
	ld B, 1			; B = crit stage
	call CalcCritMove		; calculate effect of move on crit ratio
	call CalcCritItem		; calculate effect of item on crit ratio
	
	ld A, B			; A = modified crit stage
	cp 2				; A = num btwn 1 and 5
	jr c, CalcCrit1
	jr z, CalcCrit2
	cp 3
	jr z, CalcCrit3
	cp 4
	jr z, CalcCrit4
	jr CalcCrit5			; if A >= 5, go to stage 5
	
CalcCrit1:
	ld A, 3
	jr CalcCritRand
CalcCrit2:
	ld A, 6
	jr CalcCritRand
CalcCrit3:
	ld A, 12
	jr CalcCritRand
CalcCrit4:
	ld A, 16
	jr CalcCritRand
CalcCrit5:
	ld A, 24
	
CalcCritRand:				; randomly determines crit status
	push AF
	ld B, 48
	call iRandom			; [0, 48) -> A
	pop BC				; 3 | 6 | 12 | 16 | 24 -> B
	cp B				; if A < B
	jr c, CalcCritYes		; then critical hit
	
CalcCritNo:
	xor A
	ld (CritBool), A		; (CritBool) <- 0
	ret
CalcCritYes:
	xor A
	inc A
	ld (CritBool), A		; (CritBool) <- 1
	ret
	
CalcCritMove:				; subroutine - calculate effect of move on crit ratio, input: A = move ID, B = crit, out: B = crit
	cp mBlazeKick
	jr z, CalcCritBlaze
	push HL			; save pt to detailed data
	RCall( cGetMoveEffect )	; A = move effect ID, B not affected
	pop HL
	cp qHighCrit
	ret nz
	
CalcCritHigh:				; HCHR -> +2 stages
	inc B
CalcCritBlaze:			; Blaze -> +1 stage
	inc B
	ret
	
CalcCritItem:				; subroutine - calculate effect of item on crit ratio, input: HL = pt to pkmn, B = crit, out: B = crit
	push BC
	call GetPKMNItem		; A = item ID
	pop BC
	cp iScopeLens
	ret nz
	
CalcCritScope:			; Scope lens -> +1 stage
	inc B
	ret
.endrelocate
	
; ---------------------------------------------------------------
; CalcCritDam
; Calculates critical hit damage modifier
; Input: (CritBool) = critical hit status
; Output: HL = crit modifier (1 | 2)
; ---------------------------------------------------------------

CalcCritDam:
.relocate SwapZone
	ld A, (CritBool)		; if (CritBool) == 0
	or A				; then mod = 1
	jr z, CalcCritDam1		; else, mod = 2
	
CalcCritDam2:
	ld HL, 2
	ret
	
CalcCritDam1:
	ld HL, 1
	ret
.endrelocate
	
; ---------------------------------------------------------------
; CalcEff
; Calculates type effectiveness modifier
; Input: A = Move ID
;        HL = Target PKMN
; Output: A = (TypeEff) = (0 | 1 | 2 | 4 | 8 | 16)
; ---------------------------------------------------------------

; ----------------------------------------------------------------------- ;
; Type Effectiveness Chart                                                ;
; http://pokemondb.net/type                                               ;
; ----------------------------------------------------------------------- ;
;    |NOR|FIG|FLY|POI|GRO|ROC|BUG|GHO|STE|FIR|WAT|GRA|ELE|PSY|ICE|DRA|DAR ;
; NOR|   |   |   |   |   |1/2|   | 0 |1/2|   |   |   |   |   |   |   |    ;
; FIG| 2 |   |1/2|1/2|   | 2 |1/2| 0 | 2 |   |   |   |   |1/2| 2 |   | 2  ;
; FLY|   | 2 |   |   |   |1/2| 2 |   |1/2|   |   | 2 |1/2|   |   |   |    ;
; POI|   |   |   |1/2|1/2|1/2|   |1/2| 0 |   |   | 2 |   |   |   |   |    ;
; GRO|   |   | 0 | 2 |   | 2 |1/2|   | 2 | 2 |   |1/2| 2 |   |   |   |    ;
; ROC|   |1/2| 2 |   |1/2|   | 2 |   |1/2| 2 |   |   |   |   | 2 |   |    ;
; BUG|   |1/2|1/2|1/2|   |   |   |1/2|1/2|1/2|   | 2 |   | 2 |   |   | 2  ;
; GHO| 0 |   |   |   |   |   |   | 2 |1/2|   |   |   |   | 2 |   |   |1/2 ;
; STE|   |   |   |   |   | 2 |   |   |1/2|1/2|1/2|   |1/2|   | 2 |   |    ;
; FIR|   |   |   |   |   |1/2| 2 |   | 2 |1/2|1/2| 2 |   |   | 2 |1/2|    ;
; WAT|   |   |   |   | 2 | 2 |   |   |   | 2 |1/2|1/2|   |   |   |1/2|    ;
; GRA|   |   |1/2|1/2| 2 | 2 |1/2|   |1/2|1/2| 2 |1/2|   |   |   |1/2|    ;
; ELE|   |   | 2 |   | 0 |   |   |   |   |   | 2 |1/2|1/2|   |   |1/2|    ;
; PSY|   | 2 |   | 2 |   |   |   |   |1/2|   |   |   |   |1/2|   |   | 0  ;
; ICE|   |   | 2 |   | 2 |   |   |   |1/2|1/2|1/2| 2 |   |   |1/2| 2 |    ;
; DRA|   |   |   |   |   |   |   |   |1/2|   |   |   |   |   |   | 2 |    ;
; DAR|   |1/2|   |   |   |   |   | 2 |1/2|   |   |   |   | 2 |   |   |1/2 ;
; ----------------------------------------------------------------------- ;

CalcEff:
.relocate SwapZone
	ld E, 4
	cp mRoar			; if move is Roar [or whirlwind]
	jr z, CalcEffNorm		; then norm eff
	
	ld B, A
	ld A, (TypeEff)
	cp $11				; if (TypeEff) set to $11 signal
	ld A, B			; restore A to Move ID
	jr z, CalcEffNorm		; then norm eff
	
	push HL
	RCall( cGetMoveType )	; A = move type
	pop HL
	ld E, 4			; if hit self in confusion
	cp $FF & %00011111		; then norm eff
	jr z, CalcEffNorm
	cp $FE & %00011111		; if future sight hit
	jr z, CalcEffNorm		; then norm eff
	push AF
	RCall( cGetType )
	call GetByteC
	ld B, C			; B = pkmn type 1
	inc HL
	call GetByte
	ld C, A			; C = pkmn type 2
	pop AF
	
	push AF			; calc for pkmn type 1
	push BC
	push BC
	ld H, A
	ld L, 17
	call MultHL
	ld DE, CalcEffTable
	add HL, DE
	pop BC
	ld D, 0
	ld E, B
	add HL, DE
	ld E, (HL)			; E = eff = (Table + 17 * A + B)
	pop BC
	pop AF
	ld D, A			; save A
	ld A, E
	cp eU
	jr z, CalcEffUnaffected
	ld A, B			; if B == C, then type2 has norm eff
	cp C
	jr z, CalcEffNorm
	ld A, D
	
	push DE			; calc for pkmn type 2
	push BC
	ld H, A
	ld L, 17
	call MultHL
	ld DE, CalcEffTable
	add HL, DE
	pop BC
	ld D, 0
	ld E, C
	add HL, DE
	ld A, (HL)
	pop DE
	ld D, A			; D = eff = (Table + 17 * A + C)
	cp eU
	jr z, CalcEffUnaffected
	cp eN
	jr z, CalcEffHalf
	cp eS
	jr z, CalcEff2
CalcEffNorm:				; else, norm eff
	ld A, E
	ld HL, TypeEff
	ld (HL), A
	ret
	
CalcEffHalf:
	srl E
	jr CalcEffNorm
	
CalcEff2:
	sla E
	jr CalcEffNorm

CalcEffUnaffected:			; 'It doesn't affect pkmn'
	xor A
	ld (TypeEff), A
	ret
.endrelocate
	
; ---------------------------------------------------------------
; CalcEffDam
; Calculates type effectiveness damage modifier
; Input: (TypeEff) = (0 | 1 | 2 | 4 | 8 | 16)
; Output: OP1 = type eff mod (0 | 1/4 | 1/2 | 1 | 2 | 4)
; ---------------------------------------------------------------

CalcEffDam:
.relocate SwapZone
	ld A, (TypeEff)
	or A
	jr z, CalcEffUnaffected0
	cp 1
	jr z, CalcEffNotVeryEff4
	cp 2
	jr z, CalcEffNotVeryEff
	cp 4
	jr z, CalcEffEffective
	cp 8 
	jr z, CalcEffSuperEff
	cp 16
	jr z, CalcEffSuperEff4
	; else, infinite loop?

CalcEffUnaffected0:			; 'It doesn't affect pkmn'
	b_call(_OP1Set0)
	ret
	
CalcEffNotVeryEff4:			; 'It's not very effective' (x1/4)
	ld HL, FPFourth
	rst rMov9ToOP1
	ret
	
CalcEffNotVeryEff:			; 'It's not very effective'
	ld HL, FPHalf
	rst rMov9ToOP1
	ret
	
CalcEffEffective:			; regular damage
	b_call(_OP1Set1)
	ret
	
CalcEffSuperEff:			; 'It's super effective!'
	b_call(_OP1Set2)
	ret
	
CalcEffSuperEff4:			; 'It's super effective!' (x4)
	b_call(_OP1Set4)
	ret
.endrelocate

; ---------------------------------------------------------------
; CalcSTAB
; Calculates STAB modifier
; Input: A = Move ID
;        HL = Atk PKMN
; Output: OP1 = STAB mod (1 | 1.5)
; ---------------------------------------------------------------

CalcSTAB:
.relocate SwapZone
	push HL			; A = move type
	RCall( cGetMoveType )
	pop HL
	push AF
	RCall( cGetType )
	call GetByteC
	ld B, C			; B = pkmn type 1
	inc HL
	call GetByte
	ld C, A			; C = pkmn type 2
	pop AF
	
	cp B				; if A == B
	jr z, CalcSTABYes		; then STAB
	cp C				; elif A == C
	jr z, CalcSTABYes		; then STAB
CalcSTABNo:				; else, no STAB
	b_call(_OP1Set1)
	ret
	
CalcSTABYes:
	ld HL, FPOnePtFive
	rst rMov9ToOP1
	ret
.endrelocate
	
; ---------------------------------------------------------------
; CalcRand
; Calculates random value, 217 <= R <= 255
; Output: HL = rand value
; ---------------------------------------------------------------

CalcRand:
.relocate SwapZone
	ld B, 39
	call iRandom			; 0 <= A < 39
	ld B, 217
	add A, B			; 217 <= A < 256
	ld H, 0
	ld L, A
	ret
.endrelocate

; ---------------------------------------------------------------
; CalcFocusBand
; Calculates effect of focus band (if target is wearing)
; Input: DE = damage to be dealt
;        (TargetPKMN) = target pkmn
; Output: DE = damage w/ focus band taken into account
;         (FocusBandAct) = 1 if focus band activated, 0 if not
; ---------------------------------------------------------------

CalcFocusBand:
.relocate SwapZone
	ld HL, (TargetPKMN)
	call GetPKMNItem		; A = target pkmn hold item, preserves HL, DE
	cp iFocusBand			; if holding focus band
	ret nz				; ret if not holding focus band
	call GetPKMNHP		; HL = target pkmn current HP, DE not affected
	call CpHLDE			; compare HL and DE, HL / DE unchanged
	jr z, CalcFocusBandEq	; cont if cur HP = damage
	ret nc				; ret if cur HP > damage
CalcFocusBandEq:
	push HL			; save cur HP
	push DE			; save damage
	ld B, 100
	call iRandom			; 0 <= A < 100
	pop DE
	pop HL
	cp 10				; 10% prb of focus band activating
	ret nc				; ret if A >= 10
	ex DE, HL			; otherwise
	dec DE				; DE = damage = cur HP - 1
	ld A, 1
	ld (FocusBandAct), A		; set focus band activation flag
	ret
.endrelocate

; ---------------------------------------------------------------
; CalcEndure
; Calculates effect of endure, if set
; Input: DE = damage to be dealt
;        (TargetPKMN) = target pkmn
; Output: DE = damage w/ endure taken into account
;         (EndureAct) = 1 if endure activated, 0 if not
; ---------------------------------------------------------------

CalcEndure:
.relocate SwapZone
	ld HL, (TargetPKMN)
	call GetPKMNEndure		; A = target endure status, preserves HL, DE
	ret z				; ret if A is 0, endure not set
	call GetPKMNHP		; HL = target pkmn current HP, DE not affected
	call CpHLDE			; compare HL and DE, HL / DE unchanged
	jr z, CalcEndureEq		; cont if cur HP = damage
	ret nc				; ret if cur HP > damage
CalcEndureEq:
	ex DE, HL			; otherwise
	dec DE				; DE = damage = cur HP - 1
	ld A, 1
	ld (EndureAct), A		; set endure activation flag
	ret
.endrelocate